/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Media.java,v 2447ffc4c3d1 2010/05/17 22:24:46 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.net.URL;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@Immutable
public class Media
  {
    // FIXME: use Dunlin Core ids
    public static final Key<String> AUTHOR = new Key<String>("author");
    public static final Key<String> COUNTRY = new Key<String>("country");
    public static final Key<String> LOCATION = new Key<String>("location");
    public static final Key<String> CALL_TYPE = new Key<String>("callType");
    public static final Key<Date> DATE = new Key<Date>("date");
    public static final Key<Id> ID = new Key<Id>("id");
    public static final Key<URL> URL = new Key<URL>("URL");

    private final TypeSafeMap map;

    public Media()
      {
        this(Collections.<Key<?>, Object>emptyMap());
      }

    private Media (final @Nonnull Map<Key<?>, Object> map)
      {
        this.map = new TypeSafeHashMap(map);
      }

    @Nonnull
    public <T> T get (final @Nonnull Key<T> key)
      throws NotFoundException
      {
        return map.get(key);
      }

    public boolean contains (final @Nonnull Key<?> item)
      {
        return map.containsKey(item);
      }

    @Nonnull
    public Set<Key<?>> getKeys()
      {
        return map.getKeys();
      }

    @Nonnegative
    public int getSize()
      {
        return map.getSize();
      }

    @Nonnull
    public Iterator<Entry<Key<?>, Object>> iterator()
      {
        return map.iterator();
      }

    @Nonnull
    public <T> Media with (final @Nonnull Key<T> key, final @Nonnull T value)
      {
        final Map<Key<?>, Object> asMap = map.asMap();
        asMap.put(key, value);

        return new Media(asMap);
      }
  }
