/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TypeSafeHashMap.java,v c6870fbce143 2010/05/17 22:11:41 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import it.tidalwave.util.NotFoundException;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@Immutable
public class TypeSafeHashMap implements TypeSafeMap
  {
    private final Map<Key<?>, Object> map = new HashMap<Key<?>, Object>();

    public TypeSafeHashMap (final @Nonnull Map<Key<?>, Object> map)
      {
        this.map.putAll(map);
      }

    @Nonnull
    public <T> T get (final @Nonnull Key<T> key)
      throws NotFoundException
      {
        return NotFoundException.throwWhenNull((T)map.get(key), "not found");
      }

    public boolean containsKey (final @Nonnull Key<?> key)
      {
        return map.containsKey(key);
      }

    @Nonnull
    public Set<Key<?>> getKeys()
      {
        return Collections.unmodifiableSet(map.keySet());
      }

    @Nonnegative
    public int getSize()
      {
        return map.size();
      }

    @Nonnull
    public Iterator<Entry<Key<?>, Object>> iterator()
      {
        return Collections.unmodifiableSet(map.entrySet()).iterator();
      }

    @Nonnull
    public Map<Key<?>, Object> asMap()
      {
        return new HashMap<Key<?>, Object>(map);
      }
  }
